require "import"
import "android.content.Context"
import "android.os.Vibrator"
import "android.media.ToneGenerator"
import "android.media.AudioManager"
import "android.widget.*"
import "com.androlua.*"
import "android.net.Uri"
import "android.content.Intent"
import "android.text.Html"
import "android.content.ClipData"
import "android.content.ClipboardManager"
import "android.view.accessibility.AccessibilityNodeInfo"

local context = activity or service
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
vibrator.vibrate(40)

local translationResult = ""

local languageList = {
    {"Kannada", "kn"},
    {"Tamil", "ta"},
    {"Malayalam", "ml"},
    {"English", "en"},
    {"Afrikaans", "af"},
    {"Albanian", "sq"},
    {"Amharic", "am"},
    {"Arabic", "ar"},
    {"Armenian", "hy"},
    {"Azerbaijani", "az"},
    {"Basque", "eu"},
    {"Belarusian", "be"},
    {"Bengali", "bn"},
    {"Bosnian", "bs"},
    {"Bulgarian", "bg"},
    {"Catalan", "ca"},
    {"Cebuano", "ceb"},
    {"Chichewa", "ny"},
    {"Chinese (Simplified)", "zh-CN"},
    {"Chinese (Traditional)", "zh-TW"},
    {"Corsican", "co"},
    {"Croatian", "hr"},
    {"Czech", "cs"},
    {"Danish", "da"},
    {"Dutch", "nl"},
    {"Esperanto", "eo"},
    {"Estonian", "et"},
    {"Filipino", "tl"},
    {"Finnish", "fi"},
    {"French", "fr"},
    {"Frisian", "fy"},
    {"Galician", "gl"},
    {"Georgian", "ka"},
    {"German", "de"},
    {"Greek", "el"},
    {"Gujarati", "gu"},
    {"Haitian Creole", "ht"},
    {"Hausa", "ha"},
    {"Hawaiian", "haw"},
    {"Hebrew", "he"},
    {"Hindi", "hi"},
    {"Hmong", "hmn"},
    {"Hungarian", "hu"},
    {"Icelandic", "is"},
    {"Igbo", "ig"},
    {"Indonesian", "id"},
    {"Irish", "ga"},
    {"Italian", "it"},
    {"Japanese", "ja"},
    {"Javanese", "jv"},
    {"Kazakh", "kk"},
    {"Khmer", "km"},
    {"Korean", "ko"},
    {"Kurdish", "ku"},
    {"Kyrgyz", "ky"},
    {"Lao", "lo"},
    {"Latin", "la"},
    {"Latvian", "lv"},
    {"Lithuanian", "lt"},
    {"Luxembourgish", "lb"},
    {"Macedonian", "mk"},
    {"Malagasy", "mg"},
    {"Malay", "ms"},
    {"Maltese", "mt"},
    {"Maori", "mi"},
    {"Marathi", "mr"},
    {"Mongolian", "mn"},
    {"Myanmar (Burmese)", "my"},
    {"Nepali", "ne"},
    {"Norwegian", "no"},
    {"Pashto", "ps"},
    {"Persian", "fa"},
    {"Polish", "pl"},
    {"Portuguese", "pt"},
    {"Punjabi", "pa"},
    {"Romanian", "ro"},
    {"Russian", "ru"},
    {"Samoan", "sm"},
    {"Scots Gaelic", "gd"},
    {"Serbian", "sr"},
    {"Sesotho", "st"},
    {"Shona", "sn"},
    {"Sindhi", "sd"},
    {"Sinhala", "si"},
    {"Slovak", "sk"},
    {"Slovenian", "sl"},
    {"Somali", "so"},
    {"Spanish", "es"},
    {"Sundanese", "su"},
    {"Swahili", "sw"},
    {"Swedish", "sv"},
    {"Tajik", "tg"},
    {"Telugu", "te"},
    {"Thai", "th"},
    {"Turkish", "tr"},
    {"Ukrainian", "uk"},
    {"Urdu", "ur"},
    {"Uzbek", "uz"},
    {"Vietnamese", "vi"},
    {"Welsh", "cy"},
    {"Xhosa", "xh"},
    {"Yiddish", "yi"},
    {"Yoruba", "yo"},
    {"Zulu", "zu"}
}


local items = {}
for _, languageInfo in ipairs(languageList) do
    table.insert(items, languageInfo[1])
end

function capitalizeFirstWord(str)
    return str:gsub("^%l", string.upper)
end

function addPunctuation(str)
    return str .. "."
end

function hasInternetConnection()
    local connectivityManager = context.getSystemService(Context.CONNECTIVITY_SERVICE)
    local networkInfo = connectivityManager.getActiveNetworkInfo()
    return networkInfo ~= nil and networkInfo.isConnected()
end

if not hasInternetConnection() then
    service.asyncSpeak("No internet connection. Unable to translate language")
    return false
end

local layout = {
    LinearLayout;
    layout_width = "fill";
    layout_height = "fill";
    orientation = "vertical";
    {
        LinearLayout;
        layout_width = "fill";
        layout_height = "wrap";
        orientation = "horizontal";
        gravity = "center_vertical";
        {
            TextView;
            layout_width = "0dp";
            layout_height = "wrap";
            layout_weight = 1;
            gravity = "center";
            text = "Google Translate";
        };
        {
            Button,
            id = "telegramButton";
            layout_width = "wrap";
            layout_height = "wrap";
            text = "Join our Telegram channel";
        };
    };
    {
        EditText,
        id = "searchBox";
        layout_width = "fill";
        layout_height = "wrap";
        hint = "Find the language you want to translate to...";
    };
    {
        GridView,
        id = "grid";
        numColumns = 1;
        layout_width = "fill";
        layout_height = "0dp";
        layout_weight = 1;
    },
    {
        Button,
        id = "exitButton";
        layout_width = "fill";
        layout_height = "wrap";
        text = "Exit";
    }
}

local dlg = LuaDialog(context)
dlg.View = loadlayout(layout)

local adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, items)
grid.adapter = adapter

dlg.setTitle("")
    .show()

searchBox.addTextChangedListener{
    onTextChanged = function(s, start, before, count)
        local keyword = s.toString()
        local filteredItems = {}
        for _, language in ipairs(items) do
            if language:lower():find(keyword:lower()) then
                table.insert(filteredItems, language)
            end
        end
        adapter.clear()
        adapter.addAll(filteredItems)
        adapter.notifyDataSetChanged()
    end
}

grid.onItemClick = function(l, v)
    dlg.dismiss()
    task(500, function()
        local language = v.text
        local defaultLanguage = Locale.getDefault().getLanguage()
        local languageCode
        for _, languageInfo in ipairs(languageList) do
            if languageInfo[1] == language then
                languageCode = languageInfo[2]
                break
            end
        end

        if languageCode then
            local url = "https://translate.google.com/m?sl=auto&hl=" .. defaultLanguage .. "&q=" .. Uri.encode(context.getText(node))
            url = url .. "&tl=" .. languageCode

            Http.get(url, function(status, result)
                if status == 200 then
                    local translation = tostring(Html.fromHtml(result:match('<div class="result%-container">([^<]+)</div>')))
                    translation = capitalizeFirstWord(translation)
                    translation = addPunctuation(translation)
                    translationResult = translationResult .. translation

                    local translationDialog = LuaDialog(context)
                    translationDialog.setTitle("Translation")
                        .setMessage(translation)
                        .setPositiveButton("Copy", function()
                            local clipboardManager = context.getSystemService(Context.CLIPBOARD_SERVICE)
                            local clip = ClipData.newPlainText("Translated Text", translation)
                            clipboardManager.setPrimaryClip(clip)
                            Toast.makeText(context, "Copied to clipboard", Toast.LENGTH_SHORT).show()
                            service.copy(translation)
                            translationResult = ""
                        end)
                        .setNegativeButton("Paste", function()
                            translationDialog.dismiss()
                            task(2000, function()
                                if service.execute("Paste") then
                                    local rootNode = service.getRootInActiveWindow()
                                    if rootNode then
                                        local focusedNode = rootNode.findFocus(AccessibilityNodeInfo.FOCUS_INPUT)
                                        if focusedNode then
                                            service.setText(focusedNode, translationResult)
                                        else
                                            Toast.makeText(context, "No input field is focused", Toast.LENGTH_SHORT).show()
                                        end
                                    else
                                        Toast.makeText(context, "Unable to find the active window", Toast.LENGTH_SHORT).show()
                                    end
                                else
                                    Toast.makeText(context, "Paste operation failed", Toast.LENGTH_SHORT).show()
                                end
                            end)
                        end)
                        .show()

                    service.speak("Translating text: " .. translation)
                    vibrator.vibrate(100)
                else
                    print("Sorry, an error occurred.\n These methods may be helpful:\n - Ensure you are connected to the internet.\n - Try to provide a stable internet connection.\n Note: Some elements, images, text boxes, etc., may not be translated.")
                    vibrator.vibrate(200)
                    service.playSoundTick()
                end
            end)
        else
            print("Language not supported")
            vibrator.vibrate(200)
            service.playSoundClock()
        end
    end)
end

telegramButton.onClick = function(view)
    dlg.dismiss()
    local url = "https://t.me/jieshuofiles"
    local intent = Intent(Intent.ACTION_VIEW, Uri.parse(url))
    context.startActivity(intent)
end

exitButton.onClick = function(view)
    service.playSoundTick()
    dlg.dismiss()
end